@@ -180,6 +180,8 @@ static int nagios_core_worker(const char *path)
 	}
 
 	enter_worker(sd, start_cmd);
+	free_worker_memory(WPROC_FORCE);
+	free_memory(get_global_macros());
 	return 0;
 }
 
@@ -227,6 +229,9 @@ int main(int argc, char **argv) {
 	nagios_macros *mac;
 	const char *worker_socket = NULL;
 	int i;
+#ifdef HAVE_SIGACTION
+	struct sigaction sig_action;
+#endif
 
 #ifdef HAVE_GETOPT_H
 	int option_index = 0;
@@ -342,6 +347,7 @@ int main(int argc, char **argv) {
 		printf("along with this program; if not, write to the Free Software\n");
 		printf("Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.\n\n");
 
+		cleanup();
 		exit(OK);
 		}
 
@@ -371,6 +377,7 @@ int main(int argc, char **argv) {
 		printf("the mailing lists, and commercial support options for Nagios.\n");
 		printf("\n");
 
+		cleanup();
 		exit(ERROR);
 		}
 
@@ -381,7 +388,10 @@ int main(int argc, char **argv) {
 	 */
 	config_file = nspath_absolute(argv[optind], NULL);
 	if(config_file == NULL) {
+		
 		printf("Error allocating memory.\n");
+
+		cleanup();
 		exit(ERROR);
 		}
 
@@ -392,7 +402,15 @@ int main(int argc, char **argv) {
 	 * we may encounter this signal before the other signal handlers
 	 * are set.
 	 */
+#ifdef HAVE_SIGACTION
+	sig_action.sa_sigaction = NULL;
+	sig_action.sa_handler = handle_sigxfsz;
+	sigfillset(&sig_action.sa_mask);
+	sig_action.sa_flags = SA_NODEFER|SA_RESTART;
+	sigaction(SIGXFSZ, &sig_action, NULL);
+#else
 	signal(SIGXFSZ, handle_sigxfsz);
+#endif
 
 	/*
 	 * let's go to town. We'll be noisy if we're verifying config
@@ -415,6 +433,7 @@ int main(int argc, char **argv) {
 		result = read_main_config_file(config_file);
 		if(result != OK) {
 			printf("   Error processing main config file!\n\n");
+			cleanup();
 			exit(EXIT_FAILURE);
 			}
 
@@ -424,6 +443,7 @@ int main(int argc, char **argv) {
 		/* drop privileges */
 		if((result = drop_privileges(nagios_user, nagios_group)) == ERROR) {
 			printf("   Failed to drop privileges.  Aborting.");
+			cleanup();
 			exit(EXIT_FAILURE);
 			}
 
@@ -433,6 +453,7 @@ int main(int argc, char **argv) {
 		 */
 		if (!verify_config && test_configured_paths() == ERROR) {
 			printf("   One or more path problems detected. Aborting.\n");
+			cleanup();
 			exit(EXIT_FAILURE);
 			}
 
@@ -452,11 +473,12 @@ int main(int argc, char **argv) {
 			printf("\n***> One or more problems was encountered while processing the config files...\n");
 			printf("\n");
 			printf("     Check your configuration file(s) to ensure that they contain valid\n");
-			printf("     directives and data defintions.  If you are upgrading from a previous\n");
+			printf("     directives and data definitions.  If you are upgrading from a previous\n");
 			printf("     version of Nagios, you should be aware that some variables/definitions\n");
 			printf("     may have been removed or modified in this version.  Make sure to read\n");
 			printf("     the HTML documentation regarding the config files, as well as the\n");
 			printf("     'Whats New' section to find out what has changed.\n\n");
+			cleanup();
 			exit(EXIT_FAILURE);
 			}
 
@@ -472,11 +494,12 @@ int main(int argc, char **argv) {
 			printf("\n***> One or more problems was encountered while running the pre-flight check...\n");
 			printf("\n");
 			printf("     Check your configuration file(s) to ensure that they contain valid\n");
-			printf("     directives and data defintions.  If you are upgrading from a previous\n");
+			printf("     directives and data definitions.  If you are upgrading from a previous\n");
 			printf("     version of Nagios, you should be aware that some variables/definitions\n");
 			printf("     may have been removed or modified in this version.  Make sure to read\n");
 			printf("     the HTML documentation regarding the config files, as well as the\n");
 			printf("     'Whats New' section to find out what has changed.\n\n");
+			cleanup();
 			exit(EXIT_FAILURE);
 			}
 
@@ -523,8 +546,7 @@ int main(int argc, char **argv) {
 
 		/* make valgrind shut up about still reachable memory */
 		neb_free_module_list();
-		free(config_file_dir);
-		free(config_file);
+		cleanup();
 
 		exit(result);
 		}
@@ -545,17 +567,20 @@ int main(int argc, char **argv) {
 
 		if (!nagios_binary_path) {
 			logit(NSLOG_RUNTIME_ERROR, TRUE, "Error: Unable to allocate memory for nagios_binary_path\n");
+			cleanup();
 			exit(EXIT_FAILURE);
 			}
 
 		if (!(nagios_iobs = iobroker_create())) {
 			logit(NSLOG_RUNTIME_ERROR, TRUE, "Error: Failed to create IO broker set: %s\n",
 				  strerror(errno));
+			cleanup();
 			exit(EXIT_FAILURE);
 			}
 
 		/* keep monitoring things until we get a shutdown command */
 		do {
+
 			/* reset internal book-keeping (in case we're restarting) */
 			wproc_num_workers_spawned = wproc_num_workers_online = 0;
 			caught_signal = sigshutdown = FALSE;
@@ -572,6 +597,7 @@ int main(int argc, char **argv) {
 			result = read_main_config_file(config_file);
 			if (result != OK) {
 				logit(NSLOG_CONFIG_ERROR, TRUE, "Error: Failed to process config file '%s'. Aborting\n", config_file);
+				cleanup();
 				exit(EXIT_FAILURE);
 				}
 			timing_point("Main config file read\n");
@@ -580,27 +606,8 @@ int main(int argc, char **argv) {
 			/* get program (re)start time and save as macro */
 			program_start = time(NULL);
 			my_free(mac->x[MACRO_PROCESSSTARTTIME]);
-			asprintf(&mac->x[MACRO_PROCESSSTARTTIME], "%lu", (unsigned long)program_start);
+			asprintf(&mac->x[MACRO_PROCESSSTARTTIME], "%llu", (unsigned long long)program_start);
 
-			/* drop privileges */
-			if(drop_privileges(nagios_user, nagios_group) == ERROR) {
-
-				logit(NSLOG_PROCESS_INFO | NSLOG_RUNTIME_ERROR | NSLOG_CONFIG_ERROR, TRUE, "Failed to drop privileges.  Aborting.");
-
-				cleanup();
-				exit(ERROR);
-				}
-
-			if (test_path_access(nagios_binary_path, X_OK)) {
-				logit(NSLOG_RUNTIME_ERROR, TRUE, "Error: failed to access() %s: %s\n", nagios_binary_path, strerror(errno));
-				logit(NSLOG_RUNTIME_ERROR, TRUE, "Error: Spawning workers will be impossible. Aborting.\n");
-				exit(EXIT_FAILURE);
-				}
-
-			if (test_configured_paths() == ERROR) {
-				/* error has already been logged */
-				exit(EXIT_FAILURE);
-				}
 			/* enter daemon mode (unless we're restarting...) */
 			if(daemon_mode == TRUE && sigrestart == FALSE) {
 
@@ -617,6 +624,28 @@ int main(int argc, char **argv) {
 				nagios_pid = (int)getpid();
 				}
 
+			/* drop privileges */
+			if(drop_privileges(nagios_user, nagios_group) == ERROR) {
+
+				logit(NSLOG_PROCESS_INFO | NSLOG_RUNTIME_ERROR | NSLOG_CONFIG_ERROR, TRUE, "Failed to drop privileges.  Aborting.");
+
+				cleanup();
+				exit(ERROR);
+				}
+
+			if (test_path_access(nagios_binary_path, X_OK)) {
+				logit(NSLOG_RUNTIME_ERROR, TRUE, "Error: failed to access() %s: %s\n", nagios_binary_path, strerror(errno));
+				logit(NSLOG_RUNTIME_ERROR, TRUE, "Error: Spawning workers will be impossible. Aborting.\n");
+				cleanup();
+				exit(EXIT_FAILURE);
+				}
+
+			if (test_configured_paths() == ERROR) {
+				/* error has already been logged */
+				cleanup();
+				exit(EXIT_FAILURE);
+				}
+
 			/* this must be logged after we read config data, as user may have changed location of main log file */
 			logit(NSLOG_PROCESS_INFO, TRUE, "Nagios %s starting... (PID=%d)\n", PROGRAM_VERSION, (int)getpid());
 
@@ -652,8 +681,11 @@ int main(int argc, char **argv) {
 				exit(EXIT_FAILURE);
 			}
 			timing_point("Query handler initialized\n");
+
+#ifdef ENABLE_NERD
 			nerd_init();
 			timing_point("NERD initialized\n");
+#endif
 
 			/* initialize check workers */
 			if(init_workers(num_check_workers) < 0) {
@@ -678,6 +710,7 @@ int main(int argc, char **argv) {
 				/* if we're dumping core, we must remove all dl-files */
 				if (daemon_dumps_core)
 					neb_unload_all_modules(NEBMODULE_FORCE_UNLOAD, NEBMODULE_NEB_SHUTDOWN);
+				cleanup();
 				exit(EXIT_FAILURE);
 				}
 			timing_point("Modules loaded\n");
@@ -721,6 +754,14 @@ int main(int argc, char **argv) {
 				}
 
 			timing_point("Object configuration parsed and understood\n");
+			
+#ifdef DETECT_RLIMIT_PROBLEM
+			/* lets do a quick system limit detection
+			   to determine if we're likely to run into any
+			   problems. */
+			rlimit_problem_detection(num_check_workers);
+			timing_point("Limit detection");
+#endif
 
 			/* write the objects.cache file */
 			fcache_objects(object_cache_file);
@@ -735,7 +776,7 @@ int main(int argc, char **argv) {
 			broker_program_state(NEBTYPE_PROCESS_START, NEBFLAG_NONE, NEBATTR_NONE, NULL);
 #endif
 
-			/* initialize status data unless we're starting */
+			/* initialize status data only if we're starting (no restarts) */
 			if(sigrestart == FALSE) {
 				initialize_status_data(config_file);
 				timing_point("Status data initialized\n");
@@ -795,7 +836,7 @@ int main(int argc, char **argv) {
 			/* get event start time and save as macro */
 			event_start = time(NULL);
 			my_free(mac->x[MACRO_EVENTSTARTTIME]);
-			asprintf(&mac->x[MACRO_EVENTSTARTTIME], "%lu", (unsigned long)event_start);
+			asprintf(&mac->x[MACRO_EVENTSTARTTIME], "%llu", (unsigned long long)event_start);
 
 			timing_point("Entering event execution loop\n");
 			/***** start monitoring all services *****/
@@ -809,7 +850,7 @@ int main(int argc, char **argv) {
 			qh_deinit(qh_socket_path ? qh_socket_path : DEFAULT_QUERY_SOCKET);
 
 			/* 03/01/2007 EG Moved from sighandler() to prevent FUTEX locking problems under NPTL */
-			/* 03/21/2007 EG SIGSEGV signals are still logged in sighandler() so we don't loose them */
+			/* 03/21/2007 EG SIGSEGV signals are still logged in sighandler() so we don't lose them */
 			/* did we catch a signal? */
 			if(caught_signal == TRUE) {
 
@@ -837,7 +878,10 @@ int main(int argc, char **argv) {
 			/* clean up the scheduled downtime data */
 			cleanup_downtime_data();
 
-			/* clean up the status data unless we're restarting */
+			/* clean up comment data */
+			free_comment_data();
+
+			/* clean up the status data if we are not restarting */
 			if(sigrestart == FALSE) {
 				cleanup_status_data(TRUE);
 				}
@@ -845,6 +889,7 @@ int main(int argc, char **argv) {
 			free_worker_memory(WPROC_FORCE);
 			/* shutdown stuff... */
 			if(sigshutdown == TRUE) {
+				shutdown_command_file_worker();
 				iobroker_destroy(nagios_iobs, IOBROKER_CLOSE_SOCKETS);
 				nagios_iobs = NULL;
 
@@ -852,8 +897,20 @@ int main(int argc, char **argv) {
 				logit(NSLOG_PROCESS_INFO, TRUE, "Successfully shutdown... (PID=%d)\n", (int)getpid());
 				}
 
-			/* clean up after ourselves */
-			cleanup();
+			/* try and collect any zombie processes */
+			if (sigrestart == TRUE) {
+
+				int status = 0;
+				pid_t child_pid;
+				log_debug_info(DEBUGL_PROCESS, 1, "Calling waitpid() on all children...\n");
+
+				while ((child_pid = waitpid(-1, &status, WNOHANG)) > 0) {
+
+					log_debug_info(DEBUGL_PROCESS, 2, " * child PID: (%d), status: (%d)\n", child_pid, status);
+					}
+
+				log_debug_info(DEBUGL_PROCESS, 1, "All children have been wait()ed on\n");
+				}
 
 			/* close debug log */
 			close_debug_log();
@@ -865,10 +922,7 @@ int main(int argc, char **argv) {
 			unlink(lock_file);
 
 		/* free misc memory */
-		my_free(lock_file);
-		my_free(config_file);
-		my_free(config_file_dir);
-		my_free(nagios_binary_path);
+		cleanup();
 		}
 
 	return OK;
